/******************************************************************************
 * Copyright (c) 2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
#pragma omp declare target
#ifdef _TMS320C6600
/* Check if building for the C66X device and include c6x.h for intrinsics */
#include <c6x.h>
#else 
/* Else we are building the .int.c file for the host.
 * Include the host side intrinsic simulation headers.
 * Also define TMS320C66X so that the intrinsics simulators 
 * for C66X only are included */
#define TMS320C66X
#include "c6xsim/C6xSimulator.h"
#include "c6xsim/c66_ag_intrins.h"
#include "c6xsim/c66_data_sim.h"
#endif
#pragma omp declare target end

void dotprod(float * srcA, float * srcB, float *dst, int size)
{
    #pragma omp target map(to:srcA[0:size], srcB[0:size], size) \
                       map(tofrom:dst[0:size])
    {
        int i;
        __float2_t a2, b2, dst2;

        #pragma omp parallel for private(i,a2,b2,dst2) 
        for (i = 0; i < size; i+=2) /* Assuming size is an even number */ 
        {
            a2 = _ftof2(srcA[i], srcA[i+1]);
            b2 = _ftof2(srcB[i], srcB[i+1]);
            dst2 = _dmpysp(a2,b2);

	        dst[i]      += _hif2(dst2);
            dst[i+1]    += _lof2(dst2);
        }
    }
}
