/******************************************************************************
 * Copyright (c) 2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "data_slice_id.h"

/*********************************************************************/
/* Constructor: Setup data slice ID                                  */
/*********************************************************************/
DataSliceID::DataSliceID(uintptr_t first, uintptr_t second)
{
    id_ = std::make_pair(first, second);

    if (first == 0 && second == 0)
        return;

    unsigned long long hash_first = std::hash<uintptr_t>()(first);
    unsigned long long hash_second = std::hash<uintptr_t>()(second);
    /* Using Matthew Szudzik's pairing function
     * Ref: http://szudzik.com/ElegantPairing.pdf */

    if (hash_first < hash_second)
    {
        hash_combined_ = hash_first*hash_first + hash_second;
    }else
    {
        hash_combined_ = hash_second*hash_second + hash_second + hash_first;
    }
}


/**
 * \brief Pretty print DataSliceID
 * \param os Output stream reference 
 * \param id DataSliceID reference
 * \return Reference to output stream object  
 */
std::ostream& operator<<(std::ostream& os, const DataSliceID& id)
{
    os  << "<" << id.start()
        << "," << id.end()
        << ">";
    return os;
}


